from earthdiagnostics import Utils, TempFile, cdftools
import shutil
import os
import numpy as np


class General(object):

    @staticmethod
    def vertical_mean(input_file, output_file, variable, lev_min, lev_max):
        """
        Choose vertical level in ocean, or vertically average between
        2 or more  ocean levels

        Created in  February 2012      Author : vguemas@ic3.cat
        Modified (more generic, i.e. for any input var) in December 2014
        Author : eleftheria.exarchou@ic3.cat

        :param input_file: input file name
        :param output_file: output file name  (=> 2D field  )
        :param variable: variable name
        :param lev_min: upper depth of the layer (in level number)
        :param lev_max: lower depth of the layer (in level number. If same as the upper layer, then the script just
        selects this layer. If its different, a vertical integral (weighted) between upper and lower level is computed
        """

        temp = TempFile.get()
        temp2 = TempFile.get()

        shutil.copy(input_file, temp)

        handler = Utils.nco.openCdf(temp)
        lev_min = handler.variables['lev'][lev_min]
        lev_max = handler.variables['lev'][lev_max]

        cdftools.run('cdfvertmean', input=temp, output=temp2, options=[variable, 'T', lev_min, lev_max,
                                                                       '-full', '-debug'])
        os.remove(temp)
        Utils.setminmax(temp2, '{0}_vert_mean'.format(variable))
        Utils.rename_variable(temp2, '{0}_vert_mean'.format(variable),
                              '{0}mean{1}-{2}'.format(variable, lev_min, lev_max))
        Utils.move_file(temp2, output_file)

    @staticmethod
    def interpolate3d(input_file, output_file, variable, nemo_version):
        """
        3-dimensional conservative interpolation to the regular atmospheric grid

        Created in November 2012      Author : vguemas@ic3.cat

        :param nemo_version:
        :param input_file:
        :param output_file:
        :param variable:
        :return:
        """
        temp = TempFile.get()
        shutil.copy(input_file, temp)
        if not os.path.exists('scrip_use'):
            os.link('/shared/earth/software/scripts/interpolation/scrip_use', 'scrip_use')

        cdo = Utils.cdo
        nco = Utils.nco
        handler = cdo.openCdf(temp)
        temp2 = TempFile.get()

        for lev in range(0, handler.dimensions['lev'].size):
            nco.ncks(input=temp, output=temp2, options='-O -d lev,{0} -v {1}'.format(lev, variable))
            nco.ncwa(input=temp2, output=temp2, options='-O -h -a lev')
            # link = 'rmp_{0}_to_regular_lev{1}.nc'.format(nemo_version, lev)
            # if not os.path.exists(link):
            #     os.link('/esnas/autosubmit/con_files/weigths/{0}/rmp_{0}_to_*_lev${1}.nc'.format(nemo_version, lev+1),
            #             link)
            scrip_use_in = open('scrip_use_in', 'w')
            scrip_use_in.writelines("&remap_inputs\n")
            scrip_use_in.writelines("   remap_wgt   = '/esnas/autosubmit/con_files/"
                                    "weigths/{0}/rmp_{0}_to_*_lev{1}.nc'\n".format(nemo_version, lev+1))
            scrip_use_in.writelines("   infile      = '{0}'\n".format(temp2))
            scrip_use_in.writelines("   invertlat   = FALSE\n")
            scrip_use_in.writelines("   var         = '{0}'\n".format(variable))
            scrip_use_in.writelines("   fromregular = FALSE\n")
            scrip_use_in.writelines("   outfile     = '{0}'\n".format(temp))
            scrip_use_in.writelines("/\n")
            scrip_use_in.close()

            Utils.execute_shell_command('/shared/earth/software/scripts/interpolation/scrip_use')

            nco.ncecat(input=temp2, output=temp2, options="-O -h")
            Utils.rename_variable(temp2, 'record', 'lev')
            shutil.move(temp2, 'tmp_{0:02d}.nc'.format(lev+1))
        nco.ncrcat(input='tmp_01.nc', output=temp2,
                   options='-n {0},2,1 -v {1}'.format(handler.dimensions['lev'].size, variable))
        nco.ncpdq(input=temp2, output=temp2, options='-O -h -a time,lev')
        nco.ncks(input=temp, output=temp2, options='-A -v lev')
        cdo.setgrid(',t106grid', input=temp2, output=temp2)
        if nemo_version[6:9] == '025':
            cdo.invertlatdata(input=temp2, output=temp2)
        Utils.move_file(temp2, output_file)


###############################################################################
#                                                                             #
#                    Cut a meridional or zonal section                        #
#                                                                             #
#                                                                             #
# $1 : input file                                                             #
# $2 : input var                                                              #
# $3 : Z/M   (zonal / meridional section)                                     #
# $4 : lat/lon                                                                #
# $5 : output file  ( => 2D )                                                 #
#                                                                             #
#         Created in September 2012     Author : vguemas@ic3.cat              #
#                                                                             #
###############################################################################
    @staticmethod
    def cut_section(input_file, output_file, variable, meridional, value):
        """
        Cut a meridional or zonal section

        Created in September 2012     Author : vguemas@ic3.cat
        :param input_file: input file
        :param output_file: output file  ( => 2D )
        :param variable: input var
        :param meridional: (zonal / meridional section)
        :param value: lat/lon
        :return:
        """

        nco = Utils.nco

        handler = nco.openCdf('mesh_hgr.nc')
        dimi = handler.dimensions['i'].size
        dimj = handler.dimensions['j'].size
        dimlev = handler.dimensions['lev'].size

        lon = handler.variables['lon'][:]
        lat = handler.variables['lat'][:]
        lev = handler.variables['lev'][:]
        handler.close()

        handler = nco.openCdf('mask.nc')
        mask = handler.variables['tmask'][:]
        handler.close()

        # Latitude / longitude of the zonal / meridional section
        exactpos = value
        if meridional:
            while exactpos < min(lon):
                exactpos += 360
            while exactpos > max(lon):
                exactpos -= 360
            size = dimj
        else:
            size = dimi

        # Collect the indexes defining the section

        listi = np.empty(size, dtype=int)
        listj = np.empty(size, dtype=int)

        for jpt in range(0, size):
            if meridional:
                vector = lon[jpt, :]
            else:
                vector = lat[:, jpt]
            distance = abs(vector - exactpos)
            if min(distance) < 2*360.0/dimi:
                pos = np.where(distance == min(distance))
                if meridional:
                    listi[jpt] = pos[0][0]
                    listj[jpt] = jpt
                else:
                    listi[jpt] = jpt
                    listj[jpt] = pos[0][0]
        # listi=array(dim=switch('$3','Z'=$nx-2,'M'=$ny-1))
        # listj=array(dim=switch('$3','Z'=$nx-2,'M'=$ny-1))
        # for (jpt in 1:length(listi)) {
        # vect=switch('$3','Z'=lat[jpt,],'M'=lon[,jpt+1])
        # if (min(abs(vect-exactpos))<(2*360./$nx)) {
        #   pos=sort(abs(vect-exactpos),index.return=T)\$ix[1]
        #   listi[jpt]=switch('$3','Z'=jpt+1,'M'=pos)
        #   listj[jpt]=switch('$3','Z'=pos,'M'=jpt)
        # }
        # }
        # listi=listi[is.na(listi)==F]
        # listj=listj[is.na(listj)==F]
        #
        # # Select variable at those indexes
        temp = TempFile.get()
        shutil.copy(input_file, temp)
        handler = nco.openCdf(temp)
        dimtime = handler.dimensions['time'].size
        var = np.empty([dimtime, dimlev, size], dtype=handler.variables[variable].dtype)
        for jpt in range(0, size):
            var[:, :, jpt] = handler.variables[variable][:, :, listj[jpt], listi[jpt]]
        handler.close()
        var = np.multiply(var, mask)
        # fnc1=open.ncdf('$1')
        # varout=array(dim=c(length(listi),$nz,$ntime))
        # for (jt in 1:$ntime) {
        # varin=get.var.ncdf(fnc1,'$2',start=c(1,1,1,jt),count=c($nx,$ny,$nz,1))
        # varin[which(mask<0.5)]=1e20
        # for (jpt in 1:length(listi)) {
        #   varout[jpt,,jt]=varin[listi[jpt],listj[jpt],]
        # }
        # }
        # close.ncdf(fnc1)
        # # Write the output
        # wtime=dim.def.ncdf("time","",seq(1,$ntime),unlim=TRUE)
        # dimout=array(dim=length(listi))
        # for (jpt in 1:length(listi)) {
        # dimout[jpt]=switch('$3','Z'=lon[listi[jpt],listj[jpt]],'M'=lat[listi[jpt],listj[jpt]])
        # }
        # wsec=switch('$3','Z'=dim.def.ncdf("lon","",dimout),'M'=dim.def.ncdf("lat","",dimout))
        # wdep=dim.def.ncdf("deptht","",depth)
        # wvar=var.def.ncdf("$2","",list(wsec,wdep,wtime),1e20)
        # fnc2=create.ncdf('$5',wvar)
        # put.var.ncdf(fnc2,wvar,varout)
        # close.ncdf(fnc2)
        nco.ncks(input=input_file, output=output_file, options='-h -A -v time')
